"""
    testsmanager.branches
    Representation of the branches

    Copyright (c) 2015 Pietro Albini <pietroalbini@ubuntu.com>
    Released under the GNU-AGPL v3 license
"""

import sys
import subprocess
import os
import shutil


class Branch:
    """Representation of a branch"""

    def __init__(self, manager, name, merge_request):
        self.manager = manager

        self.name = name
        self.merge_request = merge_request

        self.check_remote_status()
        self.check_local_status()

    def check_remote_status(self):
        """Check the branch merging status on GitLab"""
        if self.name in self.manager.config["keep-branches"]:
            self.active = True
            return

        result = self.manager.gitlab.merge_request(self.merge_request)
        if result:
            active = result["state"] == "opened"
        else:
            active = False
        self.active = active

    def check_local_status(self):
        """Check if the branch is present locally"""
        present_in = ["envs", "caches", "public"]
        for one in present_in:
            path = os.path.join(self.manager.root, one, self.name)
            if not os.path.exists(path):
                self.present = False
                return
        self.present = True

    def deploy(self):
        """Deploy the branch"""
        self.check_local_status()
        # Ok, branch already deployed
        if self.present:
            return

        make_dirs_in = ["envs", "caches", "public"]
        for dir in make_dirs_in:
            os.mkdir(os.path.join(self.manager.root, dir, self.name))

        commands = [
            "rm -rf %s" % self.manager.build_dir,
            "mkdir -p %s" % self.manager.build_dir,
            "cd %s && git fetch" % self.manager.git_dir,
            "git --git-dir=%s/.git --work-tree=%s checkout -f %s" %
                (self.manager.git_dir, self.manager.build_dir, self.name),
            "%s/scripts/build.sh" % self.manager.build_dir,
            "virtualenv %s/%s" % (self.manager.envs_dir, self.name),
            "cd %s && git rev-parse HEAD > %s/%s/commit" %
                (self.manager.build_dir, self.manager.envs_dir, self.name),
            "%s/%s/bin/pip install %s/out.tar.gz" % (self.manager.envs_dir,
                self.name, self.manager.build_dir),
            ("ln -s %s/%s/lib/python3.%s/site-packages/uitwww/static "
             "%s/%s/static" % (self.manager.envs_dir, self.name,
                sys.version_info[1], self.manager.public_dir, self.name)),
            "rm -rf %s" % self.manager.build_dir,
        ]

        for command in commands:
            subprocess.call(command, shell=True)

        self.manager.details["branches"][self.name] = self.merge_request
        self.manager.save_details()

    def destroy(self):
        """Destroy the local copy"""
        self.check_local_status()
        # Ok, branch not present
        if not self.present:
            return

        remove_from = ["envs", "caches", "public"]
        for dir in remove_from:
            shutil.rmtree(os.path.join(self.manager.root, dir, self.name))

        try:
            del self.manager.details["branches"][self.name]
        except KeyError:
            pass
        self.manager.save_details()
