import subprocess
import atexit
import time

from werkzeug import serving


class InstancesManager:
    """This class will manage all the running branches"""

    def __init__(self, manager, main, processor):
        self.manager = manager
        self.running = False

        self._stop = False
        self._processes = {}
        self._branches = {}

        self._main_app = main
        self._hooks_processor = processor

        atexit.register(_stop(self))

    def load_branch(self, branch):
        """Load a branch into the instances manager"""
        self._branches[branch.name] = branch

        if self.running:
            self._run_branch(branch.name)

    def remove_branch(self, name):
        """Remove a branch from the instances manager"""
        if name not in self._branches:
            return

        del self._branches[name]
        if self.running:
            self._processes[name].terminate()
            del self._processes[name]

    def run(self):
        """Run all the instances"""
        if self.running:
            raise RuntimeError("Already running!")
        self.running = True
        self._stop = False

        for branch in self._branches.values():
            self._run_branch(branch)

        # Starts also the receiver part
        self._hooks_processor.start()
        serving.run_simple("127.0.0.1", 8080, self._main_app)
        self._hooks_processor.stop = True

        # Stop all the running processes
        for process in self._processes.values():
            process.terminate()
        self._processes = {}

        print()

    def _run_branch(self, branch):
        """Run a single branch"""
        command = [
            "%s/%s/bin/uitwww" % (self.manager.envs_dir, branch.name),
            "run", "-g", "%s.py" % self.manager.gunicorn_config_file,
            "%s/%s" % (self.manager.caches_dir, branch.name),
        ]
        process = subprocess.Popen(command)
        self._processes[branch.name] = process

    def stop(self):
        """Stop the server"""
        if not self.running:
            return

        self._stop = True


def _stop(inst):
    """Function passed to atexit which stops the know world"""
    def __(*_):
        if inst.running:
            inst.stop()
    return __
